from .kmer_database import KmerDB, KmerBuilder, FastaChunkReader
from .kmerGWAS_kmer import KmerGWASDB, KmerGWASDBBuilder
from .bin.IBSpy_window_count import window_count
import argparse


def main():
	parser = argparse.ArgumentParser()
	parser.add_argument("-w", "--window_size", default=500000,
		help="window size to analyze", type=int)
	parser.add_argument("-k", "--kmer_size", default=31,
		help="Kmer size of the database", type=int)
	parser.add_argument("-d", "--database", default="./tests/data/test4B.jagger.kmerGWAS_k31",
		help="Kmer database")
	parser.add_argument("-r", "--reference", default="./tests/data/test4B.stanley.fa",
		help="The reference with the position of the kmers")
	parser.add_argument("-z", "--compress", default=False, action="store_true", 
		help="When an ouput file is present, it is compressed as .gz")
	parser.add_argument("-o", "--output", default=None, 
		help="Output file. If missing, the ouptut is sent to stdout")
	args = parser.parse_args()
	window_count(args)

if __name__ == '__main__':
	main()



import os
import sys


import logging
import operator
import unittest
from IBSpy import FastaChunkReader, KmerGWASDBBuilder


class TestFasta(unittest.TestCase):

	logger = logging.getLogger("test")
	logger.setLevel(logging.DEBUG)
	
	def setUp(self):
		self.data_path="./tests/data/"

	def test_fasta_iterator(self):
		fasta_iter = FastaChunkReader(self.data_path + "/short_test.fa", chunk_size = 50, kmer_size=31)
		#sizes = []
		seqs=[
			'ATGGGCAAGGGCAGGAATAAGAAGAAGAAGGCGCCCGCCGCCGCCGCCCC',
			'AGAAGAAGAAGGCGCCCGCCGCCGCCGCCCCCTCCCATGTAGCGCTCAAC',
			'CGCCGCCGCCCCCTCCCATGTAGCGCTCAACGGCGACGAGCCGCAGCAGG',
			'GTAGCGCTCAACGGCGACGAGCCGCAGCAGGTGGATTCGGCAGCGGCCAG',
			'AGCCGCAGCAGGTGGATTCGGCAGCGGCCAGCGGAGACGCGATT',
			'ATGGCGCCGACTGCCCAGGTACCACCTCCCTCACCAACCCCCGCTGNNNN',
			'TACCACCTCCCTCACCAACCCCCGCTGNNNNNNNNNNNNNNCAGTTATTA',
			'CCCCGCTGNNNNNNNNNNNNNNCAGTTATTAATGTGCATGCCTCATTTCC',
			'NNNCAGTTATTAATGTGCATGCCTCATTTCCTGTTTCTTGGTTCTTCAGA',
			'TGCCTCATTTCCTGTTTCTTGGTTCTTCAGATTGCGGCGCATGA']

		starts=[0,19,38, 57, 76, 0, 19, 38, 57, 76]
		ends=[50,69,88,107,120,50,69,88,107,120]
		seqnames=[
			'chr4B__ari:150492206-150497405',
			'chr4B__ari:150492206-150497405',
			'chr4B__ari:150492206-150497405',
			'chr4B__ari:150492206-150497405',
			'chr4B__ari:150492206-150497405',
			'chr4B__jag:153229422-153234621',
			'chr4B__jag:153229422-153234621',
			'chr4B__jag:153229422-153234621',
			'chr4B__jag:153229422-153234621',
			'chr4B__jag:153229422-153234621']

		i = 0
		for seq in fasta_iter:
			self.assertEqual(seqs[i], seq['seq'])
			self.assertEqual(starts[i], seq['start'])
			self.assertEqual(ends[i], seq['end'])
			self.assertEqual(seqnames[i], seq['seqname'])
			i += 1
			#print("'" + str(seq['seqname']) + "',")

	def test_fasta_to_kmers(self):

		builder = KmerGWASDBBuilder(31)
		fasta_iter = FastaChunkReader(self.data_path + "/short_test.fa", chunk_size = 50, kmer_size=31)
		sizes = [20, 20, 20, 20, 14, 16, 0, 0, 17, 14]
		i = 0
		for seq in fasta_iter: 
			kmers = builder.sequence_to_kmers(seq['seq'])
			self.assertEqual(sizes[i], len(kmers))
			i += 1
			
			





import os
import sys
#import pathlib
#add_path = os.path.dirname(os.path.abspath(__file__))+ "/.." 
#print(add_path)
#sys.path.insert(0, add_path)

import logging
import operator
import unittest
from IBSpy import KmerGWASDBBuilder, KmerGWASDB, FastaChunkReader 

class TestKmerGWAS(unittest.TestCase):

	logger = logging.getLogger("test")
	logger.setLevel(logging.DEBUG)

	def setUp(self):
		self.data_path="./tests/data/"

	def test_build_kmer(self):
		test = [
		'AAAAAAAAA',
		'TAAAAAAAA',
		'AAAAAAATA',
		'AAAAAATAA',
		'AAAAATAAA',
		'AAAATAAAA',
		'AAATAAAAA',
		'AATAAAAAA',
		'ATAAAAAAA',
		'TAAAAAAAA',
		'TTTTTTTTT',
		'GGGGGGGGG',
		'GGGGGGGGT',
		'GGGGGGGTG',
		'GGGGGGTGG',
		'GGGGGTGGG',
		'CCCCTCCCC',
		'AAATCCCCC',
		'AATCCCCCC',
		'ATCCCCCCC',
		'TCCCCACCC',
		'TTTTTTTTT',
		]
		kmer_builder = KmerGWASDBBuilder(9)
		
		for x in test:
			tested = kmer_builder.string_to_kmer(x)
			back = kmer_builder.kmer_to_string(tested)
			self.assertEqual(x, back)
			tested = kmer_builder.string_to_kmer(x.lower())
			back = kmer_builder.kmer_to_string(tested)
			self.assertEqual(x, back)
			#print (x +  "->"  + str(tested)  + "->"  + back) 
		test = [
		'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA',
		'TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT',
		'GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG',
		'CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC',
		'TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT',
		'TTTTTTTTTTTTCCCCCCCTTTTTTTTTTTT',
		'AAAAAAAAAGGGAAAGGGAAGAGGGAAAGGG',
		'AAAAAAAATGGAAAGGGAAGAGGGAAAGGGA',
		'AAAAAAATAGGAAAGGGAAGAGGGAAAGGGA',
		'AAAAAATAAGGAAAGGGAAGAGGGAAAGGGA',
		'AAAAATAAAGGAAAGGGAAGAGTTCTTGGGA',
		'AAAATAAAATTCTTGGGAAGAGTTCTTGGGA',
		'AAATAAAAAGGAAAGGGAAGAGGGAAAGGGA',
		'AATAAAAAAGGAAAGGGAAGAGGGAAAGGGA',
		'ATAAAAAAAGGAAAGGGAAGAGGGAAAGGGA',
		'TAAAAAAAAGGAAAGGGGGGCCTTAAAGGGA',
		'TTTTTTTTTGGAAAGGGGGGCCTTAAAGGGA',
		'GGGGGGGGGGGAAAGGGGGGCCTTAAAGGGA',
		'GGGGGGGGTGGAAAGGGAAGAGGGAAAGGGA',
		'GGGGGGGTGGGAAAGGGAAGAGGGAAAGGGA',
		'GGGGGGTGGGGAAAGGGAAGAGGGAAAGGGA',
		'GGGGGTGGGGGAAAGGGAAGAGGGAAAGGGA',
		'CCCCTCCCCGGAAAGGGAAGAGGGAAAGGGA',
		'AAATCCCCCGGAAAGGGAAGAGGGAAAGGGA',
		'AATCCCCCCGGAAAGGGAAGAGGGAAAGGGA',
		'ATCCCCCCCGGAAAGGGAAGAGGGAAAGGGA',
		'TCCCCACCCGGAAAGGGAAGAGGGAAAGGGA',
		'TTTTTTTTTGGAAAGGGAAGAGGGAAAGGGA'
		]

		kmer_builder2 = KmerGWASDBBuilder(31)
		
		for x in test:
			tested = kmer_builder2.string_to_kmer(x)
			back   = kmer_builder2.kmer_to_string(tested)
			#print(len(x))
			#print (x +  "->"  + str(tested)  + "->"  + back)
			self.assertEqual(x, back)

	def test_build_kmer_db(self):
		kmerdb = KmerGWASDB(31)
		kmer_builder = KmerGWASDBBuilder(31)
		kmerdb.load_from_fasta(self.data_path + "/short_test_duplicate.fa")
		#Appears twice: GGGCAGGAATAAGAAGAAGAAGGCGCCCGCC GTTTCTTGGTTCTTCAGATTGCGGCGCATGA
		#print("Size: " + str(len(kmerdb)))
		self.assertEqual(len(kmerdb), 120)

		#for i in range(0,len(kmerdb)):
			#print(kmerdb[i])
			#print(kmer_builder.kmer_to_string(kmerdb[i]))
		#	pass
		
		kmer_builder2 = KmerGWASDBBuilder(31)
		
		test = [
		"GGGCAGGAATAAGAAGAAGAAGGCGCCCGCC",
		"GTTTCTTGGTTCTTCAGATTGCGGCGCATGA",
		'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA',
		'TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT']
		results = [True, True, False, False]
		i = 0
		for x in test:
			tested = kmer_builder2.string_to_kmer(x)
			#print(x)
			self.assertEqual (tested in kmerdb ,results[i])
			i+=1
			
	def test_compare_kmers(self):
		kmer_builder = KmerGWASDBBuilder(31)
		tests = [
		'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA',
		'TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT',
		'GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG',
		'CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC',
		'AAAAAAAAAGGGAAAGGGAAGAGGGAAAGGG']
		self.assertEqual(kmer_builder.compare(tests[0], tests[1]),  0)
		self.assertEqual(kmer_builder.compare(tests[2], tests[1]),  1)
		self.assertEqual(kmer_builder.compare(tests[1], tests[2]), -1)

	def run_db_tests(self, kmerdb):
		references=['jagger', 'arinalrfor', 'julius', 'lancer', 'landmark', 'mace', 'norin61', 'spelta', 'stanley', 'sy_mattis']
		expected = [
		[2970, 2201], [2939, 2170], [2970, 2201], [2609],
		[2970, 2201], [2970, 2201], [2970, 2201], [2970, 2201],
		[2879, 2176], [2970, 2201]
		]
		i = 0
		for r in references:	
			path = self.data_path + "/test4B." + r + ".fa"
			windows = kmerdb.kmers_in_windows(path, window_size=3000)
			obs =  map(lambda w: w['observed_kmers'], windows)
			j = 0
			for o in obs:
				self.assertEqual(expected[i][j], o)
				j += 1
			i += 1



	def test_kmer_db(self):
		kmerdb = KmerGWASDB(31)
		kmerdb.load_from_fasta(self.data_path + "/test4B.jagger.fa")
		self.run_db_tests(kmerdb)
		
		saved_file = self.data_path + "test4B.jagger.temp.kmerGWAS_k31"
		kmerdb.save(saved_file)
		kmerdb_read = KmerGWASDB(31)
		kmerdb_read.load(saved_file)
		self.run_db_tests(kmerdb_read)

		saved_file = self.data_path + "test4B.jagger.kmerGWAS_k31"
		kmerdb.save(saved_file)
		kmerdb_read = KmerGWASDB(31)
		kmerdb_read.load(saved_file)
		self.run_db_tests(kmerdb_read)

if __name__ == '__main__':
	unittest.main()


cimport kmer_gwas

from libc.string cimport strcpy, strlen

cdef extern from "stdint.h":
	ctypedef unsigned long long uint64_t
	ctypedef unsigned short uint8_t

ctypedef uint64_t kmerGWAS_kmer

cdef extern from "kmer_db.h":
	struct KmerGwasTable:
		uint64_t number_of_kmers
		uint64_t capacity
		kmerGWAS_kmer * kmer
		uint8_t kmer_size
		uint8_t readonly


cdef class KmerGWAS_builder:
	cdef short _kmer_size
	cdef char * _buffer
	cdef kmerGWAS_kmer _kmer
	
	def __cinit__(self, kmer_size):
		self._kmer_size = kmer_size
		self._buffer = kmer_gwas.kmer_string_alloc(kmer_size)

	def string_to_kmer(self, sequence):
		py_byte_string = sequence.encode('UTF-8')
		strcpy(self._buffer, py_byte_string)
		self._kmer = kmer_gwas.kmer2bits(self._buffer, self._kmer_size)
		return self._kmer

	def kmer_to_string(self, kmer ):
		self._kmer = kmer
		kmer_gwas.bits2kmer31(self._kmer, self._kmer_size, self._buffer)
		return self._buffer

	def compare(self, a, b):
		return kmer_gwas.kmer_compare(a,b)

	def __dealloc__(self):
		if self._buffer is not NULL:
			kmer_gwas.kmer_string_free(self._buffer)

cdef class KmerGWAS_database: 
	cdef KmerGwasTable * _kgt

	def __cinit__(self, kmer_size): 
		self._kgt = kmer_gwas.kmer_gwas_table_new(kmer_size)

	def kmer_size(self):
		return self._kgt.kmer_size

	def add_kmers(self, sequence):
		py_byte_string = sequence.encode('UTF-8')
		kmer_gwas.kmer_gwas_table_add_kmers_from_string(py_byte_string, self._kgt)

	def read_mmap(self, filename):
		py_byte_string = filename.encode('UTF-8')
		kmer_gwas.kmer_gwas_table_mmap_read(py_byte_string, self._kgt)

	def save(self, filename):
		py_byte_string = filename.encode('UTF-8')
		kmer_gwas.kmer_gwas_table_save(py_byte_string, self._kgt)

	def sort_unique(self):
		return kmer_gwas.kmer_gwas_sort_and_filter_unique(self._kgt)

	def __contains__(self, kmer):
		ret = kmer_gwas.kmer_gwas_table_find(kmer, self._kgt) 
		return ret is not NULL

	def __len__(self):
		return self._kgt.number_of_kmers

	def  __getitem__(self, index):
		return kmer_gwas.kmer_gwas_table_get(index, self._kgt)

	def __dealloc__(self):
		kmer_gwas.kmer_gwas_table_free(&self._kgt)


